Package: SpaDES.core
Type: Package
Title: Core Utilities for Developing and Running Spatially Explicit
        Discrete Event Models
Description: Provides the core framework for a discrete event system to 
    implement a complete data-to-decisions, reproducible workflow.
    The core components facilitate the development of modular pieces, 
    and enable the user to include additional functionality by running user-built modules.
    Includes conditional scheduling, restart after interruption, packaging of
    reusable modules, tools for developing arbitrary automated workflows,
    automated interweaving of modules of different temporal resolution,
    and tools for visualizing and understanding the within-project dependencies.
URL: https://spades-core.predictiveecology.org/,
        https://github.com/PredictiveEcology/SpaDES.core
Date: 2026-05-20
Version: 3.1.2
Authors@R: c(
    person("Alex M", "Chubaty", , "achubaty@for-cast.ca", role = c("aut"),
           comment = c(ORCID = "0000-0001-7146-8135")),
    person("Eliot J B", "McIntire", , "eliot.mcintire@canada.ca", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-6914-8316")),
    person("Yong", "Luo", , "Yong.Luo@gov.bc.ca", role = "ctb"),
    person("Steve", "Cumming", , "Steve.Cumming@sbf.ulaval.ca", role = "ctb"),
    person("Ceres", "Barros", , "ceres.barros@ubc.ca", role = "ctb",
           comment = c(ORCID = "0000-0003-4036-977X")),
    person(paste("His Majesty the King in Right of Canada,",
                 "as represented by the Minister of Natural Resources Canada"), role = "cph")
  )
Depends: R (>= 4.3), quickPlot (>= 1.0.2), reproducible (>= 3.0.0)
Imports: cli, data.table (>= 1.11.0), fs, igraph (>= 1.0.1), lobstr,
        methods, qs2, Require (>= 1.0.1), rlang, stats, terra (>=
        1.7-46), tools, utils, whisker
Suggests: archive, CircStats, codetools, xml2, xmlparsedata, covr, DBI,
        DiagrammeR (>= 0.8.2), future, future.callr, ggplot2,
        ggplotify, gitcreds, googledrive, httr, httr2, knitr, lattice,
        lme4, logging, magrittr, pkgload, png, RColorBrewer (>= 1.1-2),
        raster (>= 2.5-8), rmarkdown, roxygen2, RSQLite, rstudioapi,
        sp, SpaDES.tools (>= 2.1.1), tcltk, testthat (>= 1.0.2), withr
Encoding: UTF-8
Language: en-CA
License: GPL-3
VignetteBuilder: knitr, rmarkdown
BugReports: https://github.com/PredictiveEcology/SpaDES.core/issues
ByteCompile: yes
Collate: 'module-dependencies-class.R' 'misc-methods.R' 'environment.R'
        'helpers.R' 'simList-class.R' 'times.R' 'simList-accessors.R'
        'Plots.R' 'cache.R' 'check.R' 'priority.R' 'checkpoint.R'
        'code-checking.R' 'codecheck-api.R' 'codecheck-engine.R'
        'codecheck-report.R' 'codecheck-rules.R' 'convertToPackage.R'
        'copy.R' 'debugging.R' 'downloadData.R'
        'simulation-parseModule.R' 'simulation-simInit.R' 'load.R'
        'memory-leaks.R' 'memory.R' 'modActiveBinding.R'
        'module-define.R' 'module-dependencies-methods.R'
        'module-malformed.R' 'module-param-check.R'
        'module-repository.R' 'module-template.R' 'moduleCoverage.R'
        'moduleMetadata.R' 'objectSynonyms.R' 'options.R' 'paths.R'
        'plotting-diagrams.R' 'plotting.R' 'progress.R'
        'project-template.R' 'reexports.R' 'restart.R' 'save.R'
        'saveLoadSimList.R' 'simulation-spades.R' 'spades-classes.R'
        'spades-core-deprecated.R' 'spades-core-package.R'
        'suppliedElsewhere.R' 'zzz.R'
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-20 20:27:44 UTC; emcintir
Author: Alex M Chubaty [aut] (ORCID: <https://orcid.org/0000-0001-7146-8135>),
  Eliot J B McIntire [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-6914-8316>),
  Yong Luo [ctb],
  Steve Cumming [ctb],
  Ceres Barros [ctb] (ORCID: <https://orcid.org/0000-0003-4036-977X>),
  His Majesty the King in Right of Canada, as represented by the Minister
    of Natural Resources Canada [cph]
Maintainer: Eliot J B McIntire <eliot.mcintire@canada.ca>
Repository: CRAN
Date/Publication: 2026-05-23 11:20:32 UTC
Built: R 4.6.0; ; 2026-05-23 15:11:54 UTC; unix
