# Generated by roxygen2: do not edit by hand

S3method(.unwrap,.moduleDeps)
S3method(.unwrap,.simDeps)
S3method(.unwrap,simList)
S3method(.wrap,.moduleDeps)
S3method(.wrap,.simDeps)
S3method(.wrap,simList)
S3method(P,simList)
S3method(all.equal,simList)
S3method(elapsedTime,simList)
S3method(end,simList)
S3method(makeMemoisable,simList)
S3method(objSize,simList)
S3method(rasterCreate,default)
S3method(start,simList)
S3method(time,simList)
S3method(unmakeMemoisable,simList_)
export("G<-")
export("P<-")
export("cachePath<-")
export("checkpointFile<-")
export("checkpointInterval<-")
export("completed<-")
export("current<-")
export("depends<-")
export("end<-")
export("envir<-")
export("events<-")
export("globals<-")
export("inputArgs<-")
export("inputPath<-")
export("inputs<-")
export("modulePath<-")
export("modules<-")
export("objs<-")
export("outputArgs<-")
export("outputPath<-")
export("outputs<-")
export("params<-")
export("paths<-")
export("progressInterval<-")
export("progressType<-")
export("rasterPath<-")
export("scratchPath<-")
export("start<-")
export("terraPath<-")
export("time<-")
export("times<-")
export("timeunit<-")
export(.callingModuleName)
export(.depsLoadOrder)
export(.depsPruneEdges)
export(.fileExtensions)
export(.first)
export(.highest)
export(.last)
export(.lowest)
export(.normal)
export(.parseModulePartial)
export(.paths)
export(.quickCheck)
export(.saveFileExtensions)
export(G)
export(P)
export(POM)
export(Paths)
export(Plots)
export(anyPlotting)
export(append_attr)
export(bindrows)
export(cachePath)
export(checkModule)
export(checkModuleLocal)
export(checkModuleMetadata)
export(checkObject)
export(checkParams)
export(checkTimeunit)
export(checkpointFile)
export(checkpointInterval)
export(checkpointLoad)
export(classFilter)
export(clearCacheEventsOnly)
export(codeCheckModule)
export(completed)
export(conditionalEvents)
export(convertTimeunit)
export(convertToPackage)
export(copyModule)
export(createsOutput)
export(current)
export(currentModule)
export(dNA)
export(dataPath)
export(dday)
export(defineEvent)
export(defineModule)
export(defineParameter)
export(depends)
export(depsEdgeList)
export(depsGraph)
export(dhour)
export(dmin)
export(dmonth)
export(dmonths)
export(doCallSafe)
export(doEvent)
export(doEvent.checkpoint)
export(documentation)
export(downloadData)
export(downloadModule)
export(dsecond)
export(dweek)
export(dweeks)
export(dyear)
export(dyears)
export(elapsedTime)
export(end)
export(envir)
export(eventDiagram)
export(events)
export(expectsInput)
export(experiment)
export(experiment2)
export(extractURL)
export(figurePath)
export(findObjects)
export(getMapPath)
export(getModuleVersion)
export(getPaths)
export(getSampleModules)
export(globals)
export(inSeconds)
export(inputArgs)
export(inputObjects)
export(inputPath)
export(inputs)
export(loadFiles)
export(loadSimList)
export(logPath)
export(maxTimeunit)
export(memoryUse)
export(memoryUseThisSession)
export(minTimeunit)
export(moduleCodeFiles)
export(moduleDefaults)
export(moduleDiagram)
export(moduleGraph)
export(moduleInputs)
export(moduleMetadata)
export(moduleObjects)
export(moduleOutputs)
export(moduleParams)
export(modulePath)
export(moduleVersion)
export(modules)
export(newModule)
export(newModuleCode)
export(newModuleDocumentation)
export(newModuleTests)
export(newObjectsCreated)
export(newProgressBar)
export(newProject)
export(newProjectCode)
export(noEventWarning)
export(objectDiagram)
export(objectSynonyms)
export(objs)
export(openModules)
export(outputArgs)
export(outputObjectNames)
export(outputObjects)
export(outputPath)
export(outputs)
export(packages)
export(paddedFloatToChar)
export(paramCheckOtherMods)
export(parameters)
export(params)
export(paths)
export(progressInterval)
export(progressType)
export(rasterCreate)
export(rasterPath)
export(rasterToMemory)
export(registerOutputs)
export(remoteFileSize)
export(reqdPkgs)
export(restartOrSimInitAndSpades)
export(restartR)
export(restartSpades)
export(rndstr)
export(saveFiles)
export(saveSimList)
export(saveState)
export(savedSimEnv)
export(scheduleConditionalEvent)
export(scheduleEvent)
export(scratchPath)
export(sessInfo)
export(setPaths)
export(simFile)
export(simInit)
export(simInit2)
export(simInitAndExperiment)
export(simInitAndSpades)
export(simInitAndSpades2)
export(simInitDefaults)
export(spades)
export(spades2)
export(spadesOptions)
export(spadesTimes)
export(start)
export(suppliedElsewhere)
export(terraPath)
export(times)
export(timeunit)
export(timeunits)
export(unzipSimList)
export(updateList)
export(use_gha)
export(writeEventInfo)
export(writeRNGInfo)
export(zipModule)
export(zipSimList)
exportClasses(simList)
exportMethods("G<-")
exportMethods("[")
exportMethods("cachePath<-")
exportMethods("checkpointFile<-")
exportMethods("checkpointInterval<-")
exportMethods("completed<-")
exportMethods("current<-")
exportMethods("depends<-")
exportMethods("end<-")
exportMethods("events<-")
exportMethods("globals<-")
exportMethods("inputArgs<-")
exportMethods("inputPath<-")
exportMethods("inputs<-")
exportMethods("modulePath<-")
exportMethods("objs<-")
exportMethods("outputArgs<-")
exportMethods("outputPath<-")
exportMethods("outputs<-")
exportMethods("params<-")
exportMethods("paths<-")
exportMethods("progressInterval<-")
exportMethods("progressType<-")
exportMethods("rasterPath<-")
exportMethods("scratchPath<-")
exportMethods("terraPath<-")
exportMethods("time<-")
exportMethods("times<-")
exportMethods("timeunit<-")
exportMethods(.addChangedAttr)
exportMethods(.addTagsToOutput)
exportMethods(.cacheMessage)
exportMethods(.callingModuleName)
exportMethods(.checkCacheRepo)
exportMethods(.parseElems)
exportMethods(.preDigestByClass)
exportMethods(.prepareOutput)
exportMethods(.robustDigest)
exportMethods(.tagsByClass)
exportMethods(Copy)
exportMethods(G)
exportMethods(Plot)
exportMethods(append_attr)
exportMethods(cachePath)
exportMethods(checkObject)
exportMethods(checkTimeunit)
exportMethods(checkpointFile)
exportMethods(checkpointInterval)
exportMethods(citation)
exportMethods(clearCache)
exportMethods(completed)
exportMethods(conditionalEvents)
exportMethods(copyModule)
exportMethods(createsOutput)
exportMethods(current)
exportMethods(currentModule)
exportMethods(dNA)
exportMethods(dataPath)
exportMethods(defineModule)
exportMethods(depends)
exportMethods(depsGraph)
exportMethods(documentation)
exportMethods(dweeks)
exportMethods(dyears)
exportMethods(eventDiagram)
exportMethods(events)
exportMethods(expectsInput)
exportMethods(extractURL)
exportMethods(figurePath)
exportMethods(globals)
exportMethods(initialize)
exportMethods(inputArgs)
exportMethods(inputObjects)
exportMethods(inputPath)
exportMethods(inputs)
exportMethods(keepCache)
exportMethods(logPath)
exportMethods(maxTimeunit)
exportMethods(minTimeunit)
exportMethods(moduleDiagram)
exportMethods(moduleGraph)
exportMethods(moduleInputs)
exportMethods(moduleMetadata)
exportMethods(moduleOutputs)
exportMethods(moduleParams)
exportMethods(modulePath)
exportMethods(moduleVersion)
exportMethods(newModule)
exportMethods(newModuleCode)
exportMethods(newModuleDocumentation)
exportMethods(newModuleTests)
exportMethods(newProject)
exportMethods(newProjectCode)
exportMethods(objectDiagram)
exportMethods(objs)
exportMethods(openModules)
exportMethods(outputArgs)
exportMethods(outputObjectNames)
exportMethods(outputObjects)
exportMethods(outputPath)
exportMethods(outputs)
exportMethods(packages)
exportMethods(parameters)
exportMethods(params)
exportMethods(paths)
exportMethods(progressInterval)
exportMethods(progressType)
exportMethods(rasterPath)
exportMethods(reqdPkgs)
exportMethods(scratchPath)
exportMethods(sessInfo)
exportMethods(show)
exportMethods(showCache)
exportMethods(terraPath)
exportMethods(times)
exportMethods(timeunit)
exportMethods(timeunits)
exportMethods(zipModule)
import(igraph)
import(methods)
importFrom(Require,Require)
importFrom(Require,extractVersionNumber)
importFrom(Require,getCRANrepos)
importFrom(Require,invertList)
importFrom(Require,messageVerbose)
importFrom(Require,modifyList2)
importFrom(Require,setdiffNamed)
importFrom(Require,trimVersionNumber)
importFrom(cli,ansi_strip)
importFrom(cli,bg_blue)
importFrom(cli,bg_yellow)
importFrom(cli,cli)
importFrom(cli,cli_code)
importFrom(cli,cli_text)
importFrom(cli,col_blue)
importFrom(cli,col_green)
importFrom(cli,col_magenta)
importFrom(cli,col_white)
importFrom(cli,col_yellow)
importFrom(cli,format_message)
importFrom(cli,start_app)
importFrom(cli,style_bold)
importFrom(data.table,":=")
importFrom(data.table,as.data.table)
importFrom(data.table,copy)
importFrom(data.table,data.table)
importFrom(data.table,fread)
importFrom(data.table,fwrite)
importFrom(data.table,is.data.table)
importFrom(data.table,rbindlist)
importFrom(data.table,set)
importFrom(data.table,setDF)
importFrom(data.table,setDT)
importFrom(data.table,setDTthreads)
importFrom(data.table,setattr)
importFrom(data.table,setcolorder)
importFrom(data.table,setkeyv)
importFrom(data.table,setnames)
importFrom(data.table,setorder)
importFrom(data.table,setorderv)
importFrom(data.table,shift)
importFrom(fs,path_abs)
importFrom(fs,path_common)
importFrom(fs,path_norm)
importFrom(grDevices,dev.cur)
importFrom(grDevices,dev.off)
importFrom(lobstr,obj_size)
importFrom(qs2,qs_read)
importFrom(qs2,qs_save)
importFrom(quickPlot,.objectNames)
importFrom(quickPlot,.parseElems)
importFrom(quickPlot,Plot)
importFrom(quickPlot,clearPlot)
importFrom(quickPlot,gpar)
importFrom(reproducible,.addChangedAttr)
importFrom(reproducible,.addTagsToOutput)
importFrom(reproducible,.cacheMessage)
importFrom(reproducible,.cacheMessageObjectToRetrieve)
importFrom(reproducible,.checkCacheRepo)
importFrom(reproducible,.file.move)
importFrom(reproducible,.grepSysCalls)
importFrom(reproducible,.objSizeWithTry)
importFrom(reproducible,.orderDotsUnderscoreFirst)
importFrom(reproducible,.preDigestByClass)
importFrom(reproducible,.prepareOutput)
importFrom(reproducible,.robustDigest)
importFrom(reproducible,.setSubAttrInList)
importFrom(reproducible,.sortDotsUnderscoreFirst)
importFrom(reproducible,.tagsByClass)
importFrom(reproducible,.unwrap)
importFrom(reproducible,.whereInStack)
importFrom(reproducible,.wrap)
importFrom(reproducible,Cache)
importFrom(reproducible,CacheStorageDir)
importFrom(reproducible,Checksums)
importFrom(reproducible,Copy)
importFrom(reproducible,Filenames)
importFrom(reproducible,asPath)
importFrom(reproducible,basename2)
importFrom(reproducible,checkPath)
importFrom(reproducible,clearCache)
importFrom(reproducible,compareNA)
importFrom(reproducible,extractFromCache)
importFrom(reproducible,getRelative)
importFrom(reproducible,keepCache)
importFrom(reproducible,linkOrCopy)
importFrom(reproducible,loadFromCache)
importFrom(reproducible,makeMemoisable)
importFrom(reproducible,makeRelative)
importFrom(reproducible,messageCache)
importFrom(reproducible,messageColoured)
importFrom(reproducible,messageDF)
importFrom(reproducible,normPath)
importFrom(reproducible,objSize)
importFrom(reproducible,paddedFloatToChar)
importFrom(reproducible,remapFilenames)
importFrom(reproducible,showCache)
importFrom(reproducible,tempdir2)
importFrom(reproducible,tempfile2)
importFrom(reproducible,unmakeMemoisable)
importFrom(reproducible,updateFilenameSlots)
importFrom(rlang,as_label)
importFrom(rlang,env_binding_are_lazy)
importFrom(rlang,is_quosure)
importFrom(stats,end)
importFrom(stats,na.omit)
importFrom(stats,runif)
importFrom(stats,setNames)
importFrom(stats,start)
importFrom(stats,time)
importFrom(terra,ext)
importFrom(terra,inMemory)
importFrom(terra,rast)
importFrom(terra,values)
importFrom(terra,writeRaster)
importFrom(tools,file_ext)
importFrom(tools,file_path_sans_ext)
importFrom(utils,as.person)
importFrom(utils,capture.output)
importFrom(utils,compareVersion)
importFrom(utils,download.file)
importFrom(utils,file.edit)
importFrom(utils,getFromNamespace)
importFrom(utils,getParseData)
importFrom(utils,getSrcFilename)
importFrom(utils,globalVariables)
importFrom(utils,ls.str)
importFrom(utils,modifyList)
importFrom(utils,object.size)
importFrom(utils,packageVersion)
importFrom(utils,person)
importFrom(utils,sessionInfo)
importFrom(utils,tail)
importFrom(utils,unzip)
importFrom(utils,write.table)
importFrom(utils,zip)
importFrom(whisker,whisker.render)
importMethodsFrom(quickPlot,.parseElems)
importMethodsFrom(quickPlot,Plot)
importMethodsFrom(reproducible,.addChangedAttr)
importMethodsFrom(reproducible,.addTagsToOutput)
importMethodsFrom(reproducible,.cacheMessage)
importMethodsFrom(reproducible,.checkCacheRepo)
importMethodsFrom(reproducible,.preDigestByClass)
importMethodsFrom(reproducible,.prepareOutput)
importMethodsFrom(reproducible,.robustDigest)
importMethodsFrom(reproducible,.tagsByClass)
importMethodsFrom(reproducible,Copy)
importMethodsFrom(reproducible,clearCache)
importMethodsFrom(reproducible,keepCache)
importMethodsFrom(reproducible,showCache)
