#' Classes defined in SpaDES
#'
#' These S4 classes are defined within `SpaDES`. "dot" classes are not exported and
#' are therefore intended for internal use only.
#'
#' @section Simulation classes:
#' \tabular{ll}{
#'   [simList()] \tab The `simList` class\cr
#'   [.moduleDeps()] \tab Descriptor object for specifying `SpaDES` module dependencies\cr
#'   [.simDeps()] \tab Defines all simulation dependencies for all modules within a `SpaDES` simulation\cr
#'   --------------------------- \tab ------------------------------------------------------------------------------------------ \cr
#' }
#'
#' @seealso [simInit()]
#' @name spadesClasses
#' @rdname spades-classes
#' @author Eliot McIntire and Alex Chubaty
NULL
