% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/code-checking.R
\name{.parseMessage}
\alias{.parseMessage}
\title{Prepend module name to a message}
\usage{
.parseMessage(m, problem, message)
}
\arguments{
\item{m}{module name}

\item{message}{rest of message}
}
\value{
returns \code{TRUE}; invoked for side effect of generating message with that starts with
\code{paste0(m, ":", message)}.
}
\description{
Also makes it blue.
}
\keyword{internal}
