% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/priority.R
\name{priority}
\alias{priority}
\alias{.first}
\alias{.highest}
\alias{.last}
\alias{.lowest}
\alias{.normal}
\title{Event priority}
\usage{
.first()

.highest()

.last()

.lowest()

.normal()
}
\value{
numeric of length 1.
}
\description{
Preset event priorities: 1 = first (highest); 5 = normal; 10 = last (lowest).
}
\author{
Alex Chubaty
}
