% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/environment.R
\name{.pkgEnv}
\alias{.pkgEnv}
\title{The \code{SpaDES.core} package environment}
\usage{
.pkgEnv
}
\description{
Environment used internally to store internal package objects and methods.
}
\keyword{internal}
