% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simList-accessors.R
\name{.cleanPkgs}
\alias{.cleanPkgs}
\title{strip GitHub repo info from vector of packages}
\usage{
.cleanPkgs(pkgs)
}
\description{
strip GitHub repo info from vector of packages
}
\keyword{internal}
